"""

Copyright (c) 2016, Konrad Foerstner <konrad@foerstner.org> and
Thorsten Bischler <thorsten.bischler@uni-wuerzburg.de>

Permission to use, copy, modify, and/or distribute this software for
any purpose with or without fee is hereby granted, provided that the
above copyright notice and this permission notice appear in all
copies.

THE SOFTWARE IS PROVIDED 'AS IS' AND THE AUTHOR DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE
AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.

"""


class WiggleParser(object):
    """

    Warning - this does not implement the full specification!

    """

    def entries(self, input_fh):
        track_name = None
        chrom_name = None
        span = None
        pos_value_pairs = []
        for line in input_fh:
            row = line[:-1].split()
            if len(row) == 0:
                continue
            if row[0].startswith("track"):
                track_name = self._track_name(row)
            elif row[0].startswith("variableStep"):
                if chrom_name:
                    prev_chrom_name = chrom_name
                    prev_span = span
                    prev_pos_value_pairs = pos_value_pairs
                    chrom_name = self._chrom_name(row)
                    span = self._span(row)
                    pos_value_pairs = []
                    yield(WiggleEntry(
                            track_name, prev_chrom_name, prev_span,
                            prev_pos_value_pairs))
                else:
                    chrom_name = self._chrom_name(row)
                    span = self._span(row)
            else:
                pos_value_pairs.append([int(row[0]), float(row[1])])
        yield(WiggleEntry(track_name, chrom_name, span, pos_value_pairs))

    def _chrom_name(self, row):
        return(self._attrs_and_values(row)["chrom"])

    def _span(self, row):
        return(self._attrs_and_values(row)["span"])

    def _track_name(self, row):
        return(self._attrs_and_values(row)["name"])

    def _attrs_and_values(self, row):
        attrs_and_values = {}
        for attr_and_value in row:
            if "=" not in attr_and_value:
                continue
            attr, value = attr_and_value.split("=")
            value = value.replace("\"", "")
            attrs_and_values[attr] = value
        return(attrs_and_values)


class WiggleEntry(object):

    def __init__(self, track_name, chrom_name, span, pos_value_pairs):
        self.track_name = track_name
        self.chrom_name = chrom_name
        self.span = span
        self.pos_value_pairs = pos_value_pairs
